#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
class CartaWrapper:
    template = """<!DOCTYPE html>
<html lang="en">
  <!-- Wrapper page for a CARTA session: the goal here is to minimize 3rd
   party libraries and have all the styling stuff inline, so no other
   files to worry about. Two variables will need to be filled in:

   session_end_date: UTC date session ends, 2021-09-03T22:20:14Z
   session_url:      the URL to the CARTA session
-->

  <head>
    <title>NRAO Archive CARTA Session</title>
    <style type="text/css">
      body,
      html {
        margin: 0;
        padding: 0;
        height: 100%;
        overflow: hidden;
      }

      .cartaSessionClock {
        position: absolute;
        bottom: 0;
        right: 0;
        font-weight: bold;
        padding: 10px;
        z-index: 100;
        background-color: rgb(54, 69, 79);
        border-radius: 5px;
        color: white;
        width: 30ch;
        font-family: "Courier New", monospace;
        box-shadow: rgba(6, 24, 44, 0.4) 0px 0px 0px 2px,
          rgba(6, 24, 44, 0.65) 0px 4px 6px -1px,
          rgba(255, 255, 255, 0.08) 0px 1px 0px inset;
      }
    </style>
    <script type="text/javascript">
      // ISO UTC date session ends.
      const session_end_date = "{{session_end_date}}";

      // URL to CARTA session

      const session_url = "{{carta_url}}";

      // Calculated session expiration time in MS.
      const session_expiration = new Date(session_end_date).getTime();

      function hide(elementId) {
        document.getElementById(elementId).style.display = "none";
      }

      function nuke(elementId) {
        document.getElementById(elementId).src = "about:blank";
      }

      function show(elementId) {
        document.getElementById(elementId).style.display = "inline";
      }

      function format_clock(days, hours, minutes, seconds) {
        let clockString = "";
        let daysString = `${days}d`;
        let hoursString = `${hours}h`;
        let minutesString = `${minutes}m`;
        let secondsString = `${seconds}s`;

        if (days > 0) {
          clockString = `${daysString} ${hoursString} ${minutesString} ${secondsString}`;
        } else if (hours > 0) {
          clockString = `${hoursString} ${minutesString} ${secondsString}`;
        } else if (minutes > 0) {
          clockString = `${minutesString} ${secondsString}`;
        } else {
          clockString = `${secondsString}`;
        }

        return `Session timer: ${clockString}`;
      }

      function update_clock_display(message) {
        const x = document.getElementsByClassName("cartaSessionClock");
        for (let i = 0; i < x.length; i++) {
          x[i].innerHTML = message;
        }
      }

      function pad(n, width, z) {
        z = z || "0";
        n = n + "";
        return n.length >= width
          ? n
          : new Array(width - n.length + 1).join(z) + n;
      }

      window.onload = function () {
        const session_timeout = session_expiration - new Date().getTime();
        document.getElementById("cartaFrame").src = session_url;
        setTimeout("hide('cartaContent')", session_timeout);
        setTimeout("show('expiredContent')", session_timeout);
        setTimeout("nuke('cartaContent')", session_timeout + 5);
      };

      // Update the count down every 1 second
      let x = setInterval(function () {
        // Source: https://www.w3schools.com/howto/howto_js_countdown.asp.

        // Get today's date and time.
        const now = new Date().getTime();

        // Find the distance between now and when the session expires.
        const distance = session_expiration - now;

        // Time calculations for days, hours, minutes and seconds
        const days = pad(Math.floor(distance / (1000 * 60 * 60 * 24)), 2);
        const hours = pad(
          Math.floor((distance % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60)),
          2
        );
        const minutes = pad(
          Math.floor((distance % (1000 * 60 * 60)) / (1000 * 60)),
          2
        );
        const seconds = pad(Math.floor((distance % (1000 * 60)) / 1000), 2);

        // Display the result.
        update_clock_display(format_clock(days, hours, minutes, seconds));

        // If the count down is finished, write some text.
        if (distance < 0) {
          clearInterval(x);
          update_clock_display("Session timer: EXPIRED");
        }
      }, 1000);
    </script>
  </head>
  <body>
    <div id="cartaContent" style="display: inline">
      <div class="cartaSessionClock">Session timer:</div>
      <iframe
        id="cartaFrame"
        height="100%"
        title="CARTA Image Viewer"
        width="100%"
      >
      </iframe>
    </div>
    <div id="expiredContent" style="display: none">
      <h3 style="margin: 10px">Your CARTA session has expired!</h3>
      <div class="cartaSessionClock">Session timer:</div>
    </div>
  </body>
</html>"""

    def render_template(self, carta_url: str, session_timeout_date: str):
        """
        render the wrapper template

        :param carta_url: generated carta url
        :param session_timeout_date: timeout date to set for wrapper
        :return:
        """
        return self.template.replace("{{session_end_date}}", session_timeout_date).replace("{{carta_url}}", carta_url)
